/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2005 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.adapter;

import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Connection;

/**
 * @author S. Chassande-Barrioz
 */

public class PostgresAdapter extends BasicRdbAdapter {
    private final static String POSTGRESCONCAT = "||";

    public PostgresAdapter() {
        super("postgres");
    }

    public boolean supportBatchPreparedStatement() {
        return false;
    }

    public String getSqlType(int typeCode, boolean usedInPK, int size, int scale)
            throws RdbAdapterException {
        switch (typeCode) {
        case TYPECODE_BOOLEAN:
        case TYPECODE_OBJBOOLEAN:
            return "BOOL";
        case TYPECODE_BYTE:
        case TYPECODE_OBJBYTE:
        case TYPECODE_SHORT:
        case TYPECODE_OBJSHORT:
            return "INT2";
        case TYPECODE_INT:
        case TYPECODE_OBJINT:
            return "INT4";
        case TYPECODE_LONG:
        case TYPECODE_OBJLONG:
            return "INT8";
        case TYPECODE_DOUBLE:
        case TYPECODE_OBJDOUBLE:
            return "FLOAT8";
        case TYPECODE_BYTEARRAY:
        case TYPECODE_SERIALIZED:
            return "BYTEA";
        case TYPECODE_BIGINTEGER:
        case TYPECODE_BIGDECIMAL:
            if (size != NOSIZE) {
                if (scale != NOSIZE) {
                    return "DECIMAL(" + size + ", " + scale + ")";
                } else {
                    return "DECIMAL(" + size + ")";
                }
            } else {
                return "DECIMAL";
            }
        case TYPECODE_DATE:
            return "TIMESTAMP(3)  without time zone"; //3 digit after the
                                                      // second ==> 1
                                                      // millisecond
        default:
            return super.getSqlType(typeCode, usedInPK, size, scale);
        }
    }

    public String getValueAsSQLString(boolean value) {
        return (value ? "TRUE" : "FALSE");
    }

    public String getValueAsSQLString(Object value, int typeCode) {
        if (value == null)
            return null;
        switch (typeCode) {
        case TYPECODE_BOOLEAN:
        case TYPECODE_OBJBOOLEAN:
            return getValueAsSQLString(((Boolean) value).booleanValue());
        default:
            return super.getValueAsSQLString(value, typeCode);
        }
    }

    /**
     * Postgres does not support the escape syntax
     */
    public void escapeFunctionOpen(StringBuffer sb) {
    }

    public void escapeFunctionClose(StringBuffer sb) {
    }

    public String getConcatExpression(String op1, String op2) {
        StringBuffer sb = new StringBuffer();
        return sb.append(op1).append(POSTGRESCONCAT).append(op2).toString();
    }

    public String getFirstLocateExpression(String substring, String instring) {
        return "position(" + substring + " in " + instring + ")";
    }

    public String getIndexedLocateExpression(String instring, String substring,
            String fromIndex) throws RdbAdapterException {
        return "position(" + substring + " in "
            + "substring(" + instring + " from " + fromIndex + "))";
    }

    /**
     * This calculates the size of a result set. If this feature is not
     * supported by a database or its jdbc driver, this method returns -1.
     * 
     * @param rs
     *            the result set which the size must be returned
     */
    public int fetchResultSetSize(ResultSet rs) throws SQLException {
        return -1;
    }

    public boolean existSequence(Connection connection, String seqName)
            throws SQLException {
        return existRelation(connection, seqName, new String[] { "SEQUENCE" });
    }

    // overrides existRelation for getTables with table names not supported by
    // driver
    protected boolean existRelation(Connection connection, String relName,
            String[] relationTypes) throws SQLException {
        boolean existtable = false;
        ResultSet rs = null;
        if (relationTypes == null) {
            relationTypes = new String[] { "TABLE", "VIEW" };
        }

        try {
            rs = connection.getMetaData().getTables(null, null, null,
                    relationTypes);
            while (rs.next() && !existtable) {
                existtable = relName.equalsIgnoreCase(rs.getString(3));
            }
        } finally {
            if (rs != null) {
                rs.close();
            }
        }
        return existtable;
    }

    public String getSubqueryAlias() {
        return "foo";
    }

    protected void modifySelectClauseWithRange(StringBuffer sb, boolean rangeStart,
            boolean rangeSize) {
        //do nothing to avoid the warning message
    }

    protected void modifyQueryWithRange(StringBuffer sb, boolean rangeStart,
            boolean rangeSize) {
        if (rangeStart) {
            sb.append(" OFFSET ?");
        }
        if (rangeSize) {
            sb.append(" LIMIT ?");
        }
    }

    public int getRangeParametersAtStart() {
        return RANGEATEND;
    }
	public String getArrayValueAsSQLStringBegin() {
		return "'{";
	}
	
	public String getArrayValueAsSQLStringEnd() {
		return "}'";
	}
	public String getArrayElementAsSQLString(Object elem, int elemTypeCode) {
		switch(elemTypeCode) {
		case TYPECODE_STRING:
			return elem.toString();
		default:
			return super.getArrayElementAsSQLString(elem, elemTypeCode);
		}
	}

}